@@ -15,8 +15,6 @@ use libc::iovec;
 use serde::{Deserialize, Serialize};
 use tinyvec::{tiny_vec, TinyVec};
 
-use crate::utils::intersection;
-
 use super::Iovec;
 use super::SgList;
 
@@ -26,7 +24,7 @@ use super::SgList;
 ///#          strustures live longer than the created SgData lifetime.
 ///#          This is the user responsibility taking care of
 ///#          allocating/deallocating Iovec / SgList memory.
-#[derive(Clone, Serialize, Deserialize, Debug)]
+#[derive(Clone, Default, Serialize, Deserialize, Debug)]
 pub struct SgData {
     data: TinyVec<[Bytes; 8]>,
 }
@@ -45,41 +43,51 @@ impl SgData {
     /// Moves the data contained in this `SgData` instance into `iovec`.
     /// Consumes this instance and creates a new one, based on the buffers
     /// described by `iovec` and `count`.
+    #[must_use]
     pub unsafe fn drain_into(self, iovec: *mut iovec, count: usize) -> Self {
-        let sglist = SgList::new(iovec, count);
+        let mut sglist = SgList::new(iovec, count);
         assert!(self.capacity() <= sglist.capacity());
 
-        for (buf, range) in sglist.iter_slices_range_mut() {
-            self.clone_masked(range)
-                .iter()
-                .fold(buf, |buf_slice, slice| {
-                    buf_slice[..slice.len()].copy_from_slice(slice);
-                    &mut buf_slice[slice.len()..]
-                });
+        for (mut buf, range) in sglist.iter_slices_range_mut() {
+            for slice in self.masked(range).iter() {
+                buf[..slice.len()].copy_from_slice(slice);
+                buf = &mut buf[slice.len()..]
+            }
         }
+
         Self::from_sglist(sglist)
     }
 
     /// Applies the `mask` to the current object and creates a new `SgData` object which describes
     /// only masked region
-    pub fn clone_masked(&self, mask: Range<usize>) -> Self {
+    #[must_use]
+    pub fn masked(&self, mask: Range<usize>) -> Self {
         let data = self
             .data
             .iter()
             .zip(self.iter_ranges())
             .filter_map(move |(bytes, range)| {
-                let intersect = intersection(&range, &mask)?;
-                let start = intersect.start - range.start;
-                let end = intersect.end - range.start;
-                Some(bytes.slice(start..end))
+                let start = mask.start.checked_sub(range.start).unwrap_or_default();
+                let end = range.end.min(mask.end).checked_sub(range.start)?;
+                let range = start..end;
+                if range.is_empty() {
+                    None
+                } else {
+                    Some(bytes.slice(range))
+                }
             })
             .collect();
 
         Self { data }
     }
 
+    #[must_use]
+    pub fn clone_masked(&self, mask: Range<usize>) -> Self {
+        self.masked(mask)
+    }
+
     pub unsafe fn from_sglist(sglist: SgList) -> Self {
-        let data = sglist.iter_static_slices().map(From::from).collect();
+        let data = sglist.iter_static_slices().map(Bytes::from).collect();
 
         Self { data }
     }
@@ -89,7 +97,7 @@ impl SgData {
         self.data.iter().map(AsRef::as_ref)
     }
 
-    // Iterate over `slices` `ranges`
+    /// Iterate over `slices` `ranges`
     fn iter_ranges(&self) -> impl Iterator<Item = Range<usize>> + '_ {
         let mut offset = 0;
         self.iter().map(move |slice| {
@@ -121,10 +129,8 @@ impl SgData {
 
     /// Create an `SgData` with specified capacity
     pub fn with_capacity(capacity: usize) -> Self {
-        let mut vec = Vec::<u8>::with_capacity(capacity);
-        unsafe {
-            vec.set_len(capacity);
-        };
+        let ptr = std::mem::ManuallyDrop::new(Vec::<u8>::with_capacity(capacity)).as_mut_ptr();
+        let vec = unsafe { Vec::from_raw_parts(ptr, capacity, capacity) };
         vec.into()
     }
 }
@@ -132,7 +138,7 @@ impl SgData {
 impl From<Vec<u8>> for SgData {
     fn from(vec: Vec<u8>) -> Self {
         Self {
-            data: tiny_vec![vec.into_iter().collect()],
+            data: tiny_vec![vec.into()],
         }
     }
 }
@@ -140,7 +146,7 @@ impl From<Vec<u8>> for SgData {
 impl From<Vec<Vec<u8>>> for SgData {
     fn from(sgvec: Vec<Vec<u8>>) -> Self {
         Self {
-            data: sgvec.into_iter().map(From::from).collect(),
+            data: sgvec.into_iter().map(Bytes::from).collect(),
         }
     }
 }
@@ -185,23 +191,13 @@ impl Extend<u8> for SgData {
     }
 }
 
-impl<'a> Extend<&'a [u8]> for SgData {
-    fn extend<T>(&mut self, iter: T)
-    where
-        T: IntoIterator<Item = &'a [u8]>,
-    {
-        self.data
-            .extend(iter.into_iter().map(|slice| Bytes::from(slice.to_vec())))
-    }
-}
-
 impl Extend<Self> for SgData {
     fn extend<T>(&mut self, iter: T)
     where
         T: IntoIterator<Item = Self>,
     {
         for item in iter {
-            self.data.extend(item.data);
+            self.data.extend(item.data)
         }
     }
 }
@@ -244,14 +240,6 @@ impl PartialEq for SgData {
     }
 }
 
-impl Default for SgData {
-    fn default() -> Self {
-        Self {
-            data: TinyVec::default(),
-        }
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -311,8 +299,8 @@ mod tests {
             0..usize::MAX
         };
 
-        let copy_a = data_a.clone_masked(mask.clone());
-        let copy_b = data_a.clone_masked(mask.clone());
+        let copy_a = data_a.masked(mask.clone());
+        let copy_b = data_a.masked(mask.clone());
         assert_eq!(copy_a, copy_b);
 
         let ser_a = serialize(&copy_a).unwrap();
@@ -321,24 +309,8 @@ mod tests {
         let deser_b: SgData = deserialize(ser_b.as_slice()).unwrap();
         assert_eq!(deser_a, deser_b);
 
-        // test converting to a vector deep-copy - i.e cloning.
-        assert_eq!(
-            data_a.clone_masked(mask.clone()),
-            data_b.clone_masked(mask.clone())
-        );
-
         unsafe {
-            // test deep-copy - i.e cloning.
-            let ser_a = serialize(&data_a.clone_masked(mask.clone())).unwrap();
-            let deser_a: SgData = deserialize(ser_a.as_slice()).unwrap();
-            let ser_b = serialize(&data_b.clone_masked(mask.clone())).unwrap();
-            let deser_b: SgData = deserialize(ser_b.as_slice()).unwrap();
-            assert_eq!(deser_a, deser_b);
-
-            drain_data_compare(
-                data_a,
-                partition_vec(vec![42; data_a.capacity()].as_slice()),
-            );
+            drain_data_compare(data_a, partition_vec(&vec![42; data_a.capacity()]));
 
             // create a vector of vector where one is of size 1
             drain_data_compare(data_a, vec![vec![42; data_a.capacity() - 1], vec![44; 1]]);
@@ -348,49 +320,39 @@ mod tests {
 
             //Drain data into masked sgvec - add compare values
             drain_data_compare(
-                &data_a.clone_masked(mask.clone()),
-                partition_vec(vec![43; data_a.clone_masked(mask.clone()).capacity()].as_slice()),
+                &data_a.masked(mask.clone()),
+                partition_vec(&vec![43; data_a.masked(mask.clone()).capacity()]),
             );
 
-            assert_eq!(
-                data_a.clone_masked(mask.clone()),
-                data_b.clone_masked(mask.clone())
-            );
+            assert_eq!(data_a.masked(mask.clone()), data_b.masked(mask.clone()));
 
-            let tmp = data_b.clone_masked(mask.clone());
+            let tmp = data_b.masked(mask.clone());
             let data: SgData = tmp.iter().map(Iovec::from).collect();
 
-            assert_eq!(data, data_a.clone_masked(mask));
+            assert_eq!(data, data_a.masked(mask));
             drop(tmp)
         }
     }
 
     // given two value-equal mutable instances of SgData.
     // Test that they are (not)-eqaul after mutation.
-    fn compare_data(data_a: &mut SgData, data_b: &mut SgData) {
+    fn compare_data(mut data_a: SgData, mut data_b: SgData) {
         // Generate two sg-datas of different types with equal byte-wise values.
-        eq_data(data_a, data_b, None);
+        eq_data(&data_a, &data_b, None);
 
         let capacity = data_a.capacity();
 
         // Define an arbitray data representation for sg-data
         let vec_def = gen_def_data(25, 25);
 
-        let assign_a = build_data(vec_def.as_slice());
-        let assign_b = build_data(vec_def.as_slice());
+        let assign_a = build_data(&vec_def);
+        let assign_b = build_data(&vec_def);
 
         data_a.add_assign(assign_a);
 
         assert_ne!(data_a, data_b);
 
-        eq_data(
-            data_a,
-            data_b,
-            Range {
-                start: 0,
-                end: capacity,
-            },
-        );
+        eq_data(&data_a, &data_b, 0..capacity);
 
         data_b.add_assign(assign_b);
 
@@ -400,35 +362,20 @@ mod tests {
         if !vec.is_empty() {
             vec[0] = if u8::MAX == vec[0] { 0 } else { vec[0] + 1 };
 
-            let data_c = &SgData::from(vec);
+            let data_c = SgData::from(vec);
             assert_ne!(data_a, data_c);
 
-            eq_data(
-                data_a,
-                data_c,
-                Range {
-                    start: 1,
-                    end: data_a.capacity(),
-                },
-            );
+            eq_data(&data_a, &data_c, 1..data_a.capacity());
 
             assert_ne!(data_b, data_c);
-            eq_data(
-                data_b,
-                data_c,
-                Range {
-                    start: 1,
-                    end: data_b.capacity(),
-                },
-            );
+            eq_data(&data_b, &data_c, 1..data_b.capacity());
         }
 
-        let data_a: &SgData = data_a;
         let sgvec = vec![vec![0_u8; data_a.capacity()]];
         let (base, count) = create_sglist(sgvec).into_inner();
 
         let drained_data = unsafe { data_a.clone().drain_into(base as *mut libc::iovec, count) };
-        assert_eq!(data_a, &drained_data);
+        assert_eq!(data_a, drained_data);
     }
 
     // Given a vector defining a Sg-data - pairs of values X size, create a random SgData instance.
@@ -440,7 +387,7 @@ mod tests {
             .map(|vec| match rng.gen_range(0..3) {
                 0 => SgData::from(Iovec::from(vec_into_iovec(vec))),
                 1 => SgData::from(vec),
-                _ => unsafe { SgData::from_sglist(create_sglist(partition_vec(vec.as_slice()))) },
+                _ => unsafe { SgData::from_sglist(create_sglist(partition_vec(&vec))) },
             })
             .collect()
     }
@@ -463,24 +410,14 @@ mod tests {
 
     fn partition_vec(vec: &[u8]) -> Vec<Vec<u8>> {
         use rand::{thread_rng, Rng};
-        let indices: Vec<usize> = vec
-            .iter()
-            .enumerate()
-            .map(|(index, _)| index)
-            .filter(|index| {
-                if *index == 0 {
-                    true
-                } else {
-                    thread_rng().gen_bool(0.02)
-                }
-            })
+        let indices: Vec<usize> = (0..vec.len())
+            .filter(|&index| index == 0 || thread_rng().gen_bool(0.02))
             .collect();
 
         indices
             .iter()
-            .cloned()
-            .zip(indices.iter().cloned().skip(1).chain(once(vec.len())))
-            .map(|(start, end)| vec[start..end].into())
+            .zip(indices.iter().skip(1).chain(once(&vec.len())))
+            .map(|(&start, &end)| vec[start..end].into())
             .collect()
     }
 
@@ -492,15 +429,7 @@ mod tests {
         for (num_of_entries, max_entry_capacity) in num_of_entries_iter.zip(max_entry_capacity_iter)
         {
             let data_def = gen_def_data(num_of_entries, max_entry_capacity);
-            println!("data def {:?}", data_def);
-
-            let mut data_a = build_data(data_def.as_slice());
-            println!("\ndata a {:?}", data_a);
-
-            let mut data_b = build_data(data_def.as_slice());
-            println!("\ndata b {:?}", data_b);
-
-            compare_data(&mut data_a, &mut data_b);
+            compare_data(build_data(&data_def), build_data(&data_def));
         }
     }
 
@@ -510,7 +439,7 @@ mod tests {
         let vec_def: Vec<(u8, usize)> = vec![(0, 44), (0, 511), (0, 44)];
 
         // define the data0 to be a vector of vectors
-        let mut data_a = SgData::from(
+        let data_a = SgData::from(
             vec_def
                 .iter()
                 .map(|(val, size)| vec![*val; *size])
@@ -527,9 +456,9 @@ mod tests {
             iov_len: vec.len(),
         });
 
-        let mut data_b = SgData::from(iovec);
+        let data_b = SgData::from(iovec);
 
-        compare_data(&mut data_a, &mut data_b);
+        compare_data(data_a, data_b);
     }
 
     #[test]
@@ -544,7 +473,7 @@ mod tests {
                 .collect::<Vec<_>>(),
         );
 
-        let mut data_combined = data0.clone_masked(0..usize::MAX);
+        let mut data_combined = data0.masked(0..usize::MAX);
 
         let vec1 = vec_def
             .iter()
@@ -554,8 +483,8 @@ mod tests {
         // define the data1 to be a direct vector
         let data1: SgData = SgData::from(vec1);
 
-        let data1_elements = data1.clone_masked(0..usize::MAX);
-        data_combined.extend(data1_elements.iter());
+        let data1_elements = data1.masked(0..usize::MAX);
+        data_combined.extend(Some(data1_elements));
 
         let mut vec_copied = data0.to_vec();
         vec_copied.extend(data1.to_vec());
