@@ -29,9 +29,10 @@ pub(crate) unsafe fn iovec_as_slice_mut(iov: &iovec) -> &mut [u8] {
 
 /// Returns a possible overlap between two ranges
 pub(crate) fn intersection(r1: &Range<usize>, r2: &Range<usize>) -> Option<Range<usize>> {
-    if r1.end <= r2.start || r1.start >= r2.end {
+    let r = r1.start.max(r2.start)..r1.end.min(r2.end);
+    if r.is_empty() {
         None
     } else {
-        Some(r1.start.max(r2.start)..r1.end.min(r2.end))
+        Some(r)
     }
 }
